@@ -20,6 +20,23 @@ from .data_source.relational_database import get_sql_data
 from .data_source.data_file import get_file_data
 
 
+def _preprocess_data(dataframe, mapping_rule, references, config):
+    # deal with ORACLE
+    if mapping_rule['source_type'] == RDB:
+        if config.get_database_url(mapping_rule['source_name']).startswith(ORACLE):
+            dataframe = normalize_oracle_identifier_casing(dataframe, references)
+
+        # for RDB NULLS are removed on query time, but the dataframe is not converted to str
+        dataframe = dataframe_columns_to_str(dataframe)
+
+    # remove NULLS for those data formats that do not allow to remove them in reading time
+    if config.apply_na_filter():
+        if mapping_rule['source_type'] in [PARQUET, FEATHER, ORC, STATA, SAS, SPSS, JSON, XML]:
+            dataframe = remove_null_values_from_dataframe(dataframe, config)
+
+    return dataframe
+
+
 def _get_references_in_mapping_rule(mapping_rule, only_subject_map=False):
     references = []
     if pd.notna(mapping_rule['subject_template']):
@@ -48,19 +65,20 @@ def _materialize_template(results_df, template, config, columns_alias='', termty
                           datatype=''):
     references = get_references_in_template(str(template))
 
-    if str(termtype).strip() == R2RML_LITERAL:
-        results_df['triple'] = results_df['triple'] + '"'
-    else:
+    if str(termtype).strip() == R2RML_IRI:
         results_df['triple'] = results_df['triple'] + '<'
+    elif str(termtype).strip() == R2RML_LITERAL:
+        results_df['triple'] = results_df['triple'] + '"'
+    elif str(termtype).strip() == R2RML_BLANK_NODE:
+        results_df['triple'] = results_df['triple'] + '_:'
 
     for reference in references:
         results_df['reference_results'] = results_df[columns_alias + reference]
 
         if config.only_write_printable_characters():
-            results_df['reference_results'] = results_df['reference_results'].apply(lambda x: remove_non_printable_characters(encode(x)))
+            results_df['reference_results'] = results_df['reference_results'].apply(lambda x: remove_non_printable_characters(x))
 
         if str(termtype).strip() == R2RML_IRI:
-            # falcon's encode is faster than urllib's quote
             results_df['reference_results'] = results_df['reference_results'].apply(lambda x: encode(x))
         elif str(termtype).strip() == R2RML_LITERAL:
             results_df['reference_results'] = results_df['reference_results'].str.replace('"', '\\"', regex=False).str.replace('\\', '\\\\"', regex=False)
@@ -72,7 +90,9 @@ def _materialize_template(results_df, template, config, columns_alias='', termty
         # add what remains in the template after the last reference
         results_df['triple'] = results_df['triple'] + template
 
-    if str(termtype).strip() == R2RML_LITERAL:
+    if str(termtype).strip() == R2RML_IRI:
+        results_df['triple'] = results_df['triple'] + '> '
+    elif str(termtype).strip() == R2RML_LITERAL:
         results_df['triple'] = results_df['triple'] + '"'
         if pd.notna(language_tag):
             results_df['triple'] = results_df['triple'] + '@' + language_tag + ' '
@@ -80,8 +100,8 @@ def _materialize_template(results_df, template, config, columns_alias='', termty
             results_df['triple'] = results_df['triple'] + '^^<' + datatype + '> '
         else:
             results_df['triple'] = results_df['triple'] + ' '
-    else:
-        results_df['triple'] = results_df['triple'] + '> '
+    elif str(termtype).strip() == R2RML_BLANK_NODE:
+        results_df['triple'] = results_df['triple'] + ' '
 
     return results_df
 
@@ -91,13 +111,9 @@ def _materialize_reference(results_df, reference, config, columns_alias='', term
     results_df['reference_results'] = results_df[columns_alias + str(reference)]
 
     if config.only_write_printable_characters():
-        results_df['reference_results'] = results_df['reference_results'].apply(lambda x: remove_non_printable_characters(encode(x)))
+        results_df['reference_results'] = results_df['reference_results'].apply(lambda x: remove_non_printable_characters(x))
 
-    if str(termtype).strip() == R2RML_IRI:
-        # falcon's encode is faster than urllib's quote
-        results_df['reference_results'] = results_df['reference_results'].apply(lambda x: encode(x))
-        results_df['triple'] = results_df['triple'] + '<' + results_df['reference_results'] + '> '
-    elif str(termtype).strip() == R2RML_LITERAL:
+    if str(termtype).strip() == R2RML_LITERAL:
         results_df['reference_results'] = results_df['reference_results'].str.replace('"', '\\"', regex=False).str.replace('\\', '\\\\"', regex=False)
         results_df['triple'] = results_df['triple'] + '"' + results_df['reference_results'] + '"'
         if pd.notna(language_tag):
@@ -106,12 +122,20 @@ def _materialize_reference(results_df, reference, config, columns_alias='', term
             results_df['triple'] = results_df['triple'] + '^^<' + datatype + '> '
         else:
             results_df['triple'] = results_df['triple'] + ' '
+    elif str(termtype).strip() == R2RML_IRI:
+        results_df['reference_results'] = results_df['reference_results'].apply(lambda x: encode(x))
+        results_df['triple'] = results_df['triple'] + '<' + results_df['reference_results'] + '> '
+    elif str(termtype).strip() == R2RML_BLANK_NODE:
+        results_df['triple'] = results_df['triple'] + '_:' + results_df['reference_results'] + ' '
 
     return results_df
 
 
 def _materialize_constant(results_df, constant, termtype=R2RML_IRI, language_tag='', datatype=''):
-    if str(termtype).strip() == R2RML_LITERAL:
+    complete_constant = ''
+    if str(termtype).strip() == R2RML_IRI:
+        complete_constant = '<' + str(constant) + '> '
+    elif str(termtype).strip() == R2RML_LITERAL:
         complete_constant = '"' + constant + '"'
 
         if pd.notna(language_tag):
@@ -120,8 +144,8 @@ def _materialize_constant(results_df, constant, termtype=R2RML_IRI, language_tag
             complete_constant = complete_constant + '^^<' + datatype + '> '
         else:
             complete_constant = complete_constant + ' '
-    else:
-        complete_constant = '<' + str(constant) + '> '
+    elif str(termtype).strip() == R2RML_BLANK_NODE:
+        complete_constant = '_:' + str(constant) + ' '
 
     results_df['triple'] = results_df['triple'] + complete_constant
 
@@ -168,7 +192,7 @@ def _materialize_mapping_rule_terms(results_df, mapping_rule, config):
     elif pd.notna(mapping_rule['subject_reference']):
         results_df = _materialize_reference(results_df, mapping_rule['subject_reference'], config, termtype=mapping_rule['subject_termtype'])
     if pd.notna(mapping_rule['predicate_template']):
-        results_df = _materialize_template(results_df, mapping_rule['predicate_template'])
+        results_df = _materialize_template(results_df, mapping_rule['predicate_template'], config)
     elif pd.notna(mapping_rule['predicate_constant']):
         results_df = _materialize_constant(results_df, mapping_rule['predicate_constant'])
     elif pd.notna(mapping_rule['predicate_reference']):
@@ -191,22 +215,6 @@ def _materialize_mapping_rule_terms(results_df, mapping_rule, config):
     return set(results_df['triple'])
 
 
-def _materalize_push_down_sql_join(mapping_rule, parent_triples_map_rule, references, parent_references, config):
-    for key, join_condition in eval(mapping_rule['join_conditions']).items():
-        parent_references.add(join_condition['parent_value'])
-        references.add(join_condition['child_value'])
-
-    triples_rule = set()
-    result_chunks = get_sql_data(config, mapping_rule, references, parent_triples_map_rule, parent_references)
-    for query_results_chunk_df in result_chunks:
-        query_results_chunk_df = dataframe_columns_to_str(query_results_chunk_df)
-        # query_results_chunk_df.dropna(axis=0, how='any', inplace=True)
-        triples_rule.update(
-            _materialize_join_mapping_rule_terms(query_results_chunk_df, mapping_rule, parent_triples_map_rule, config))
-
-    return triples_rule
-
-
 def _merge_results_chunks(query_results_chunk_df, parent_query_results_chunk_df, mapping_rule):
     child_join_references, parent_join_references = get_references_in_join_condition(mapping_rule)
 
@@ -232,54 +240,40 @@ def _materialize_mapping_rule(mapping_rule, subject_maps_df, config):
             subject_maps_df[subject_maps_df.triples_map_id == mapping_rule['object_parent_triples_map']].iloc[0]
         parent_references = _get_references_in_mapping_rule(parent_triples_map_rule, only_subject_map=True)
 
-        if config.push_down_sql_joins() and \
-                mapping_rule['source_type'] == RDB_SOURCE_TYPE and \
-                mapping_rule['source_name'] == parent_triples_map_rule['source_name']:
-            triples.update(_materalize_push_down_sql_join(mapping_rule, parent_triples_map_rule, references,
-                                                          parent_references, config))
+        # add references used in the join condition
+        references, parent_references = add_references_in_join_condition(mapping_rule, references, parent_references)
 
-        else:
-            # add references used in the join condition (not needed when pushing down join to SQL)
-            references, parent_references = add_references_in_join_condition(mapping_rule, references,
-                                                                                   parent_references)
-
-            if mapping_rule['source_type'] == RDB_SOURCE_TYPE:
-                result_chunks = get_sql_data(config, mapping_rule, references)
-            elif mapping_rule['source_type'] in FILE_SOURCE_TYPES:
-                result_chunks = get_file_data(config, mapping_rule, references)
-
-            for query_results_chunk_df in result_chunks:
-                query_results_chunk_df = dataframe_columns_to_str(query_results_chunk_df)
-                #query_results_chunk_df.replace(config.get_na_values(), np.NaN)
-                query_results_chunk_df.dropna(axis=0, how='any', inplace=True)
-                query_results_chunk_df = query_results_chunk_df.add_prefix('child_')
-
-                if parent_triples_map_rule['source_type'] == RDB_SOURCE_TYPE:
-                    parent_result_chunks = get_sql_data(config, parent_triples_map_rule, parent_references)
-                elif parent_triples_map_rule['source_type'] in FILE_SOURCE_TYPES:
-                    parent_result_chunks = get_file_data(config, parent_triples_map_rule, parent_references)
-
-                for parent_query_results_chunk_df in parent_result_chunks:
-                    parent_query_results_chunk_df = dataframe_columns_to_str(parent_query_results_chunk_df)
-                    #parent_query_results_chunk_df.replace(config.get_na_values(), np.NaN)
-                    parent_query_results_chunk_df.dropna(axis=0, how='any', inplace=True)
-                    parent_query_results_chunk_df = parent_query_results_chunk_df.add_prefix('parent_')
-                    merged_query_results_chunk_df = _merge_results_chunks(query_results_chunk_df,
-                                                                          parent_query_results_chunk_df, mapping_rule)
-
-                    triples.update(_materialize_join_mapping_rule_terms(merged_query_results_chunk_df, mapping_rule,
-                                   parent_triples_map_rule))
-
-    else:
-        if mapping_rule['source_type'] in RDB_SOURCE_TYPE:
+        if mapping_rule['source_type'] == RDB:
             result_chunks = get_sql_data(config, mapping_rule, references)
         elif mapping_rule['source_type'] in FILE_SOURCE_TYPES:
             result_chunks = get_file_data(config, mapping_rule, references)
 
         for query_results_chunk_df in result_chunks:
-            #query_results_chunk_df.replace(config.get_na_values(), np.NaN)
-            query_results_chunk_df.dropna(axis=0, how='any', inplace=True)  # this line should apply only in some cases
-            query_results_chunk_df = dataframe_columns_to_str(query_results_chunk_df)
+            query_results_chunk_df = _preprocess_data(query_results_chunk_df, mapping_rule, references, config)
+            query_results_chunk_df = query_results_chunk_df.add_prefix('child_')
+
+            if parent_triples_map_rule['source_type'] == RDB:
+                parent_result_chunks = get_sql_data(config, parent_triples_map_rule, parent_references)
+            elif parent_triples_map_rule['source_type'] in FILE_SOURCE_TYPES:
+                parent_result_chunks = get_file_data(config, parent_triples_map_rule, parent_references)
+
+            for parent_query_results_chunk_df in parent_result_chunks:
+                parent_query_results_chunk_df = _preprocess_data(parent_query_results_chunk_df, mapping_rule, references, config)
+                parent_query_results_chunk_df = parent_query_results_chunk_df.add_prefix('parent_')
+                merged_query_results_chunk_df = _merge_results_chunks(query_results_chunk_df,
+                                                                      parent_query_results_chunk_df, mapping_rule)
+
+                triples.update(_materialize_join_mapping_rule_terms(merged_query_results_chunk_df, mapping_rule,
+                                                                    parent_triples_map_rule, config))
+
+    else:
+        if mapping_rule['source_type'] in RDB:
+            result_chunks = get_sql_data(config, mapping_rule, references)
+        elif mapping_rule['source_type'] in FILE_SOURCE_TYPES:
+            result_chunks = get_file_data(config, mapping_rule, references)
+
+        for query_results_chunk_df in result_chunks:
+            query_results_chunk_df = _preprocess_data(query_results_chunk_df, mapping_rule, references, config)
             triples.update(_materialize_mapping_rule_terms(query_results_chunk_df, mapping_rule, config))
 
     return triples
